#!/usr/bin/env python
# coding: utf-8

# In[4]:


import pandas as pd
import numpy as np
import jieba
from sklearn.feature_extraction.text import CountVectorizer, TfidfVectorizer
from sklearn.pipeline import Pipeline
from sklearn.svm import SVC
from sklearn.model_selection import cross_validate
# from sklearn.preprocessing import MaxAbsScaler
import csv
# import dask_ml.model_selection as dcv
# from sklearn.model_selection import train_test_split
# from dask.distributed import Client
from sklearn.model_selection import train_test_split
from sklearn.metrics import recall_score, precision_score, roc_auc_score, accuracy_score
# from sklearn.metrics import classification_report
# from joblib import dump, load
# import dill as pickle
# import pickle
# from sklearn.metrics import classification_report
# set seed
np.random.seed(0)

# set path to save csv results
score_path = '../data/scores/humil_scores.csv'
cv_score_path = '../data/scores/humil_cv_scores.csv'

# for parallelization

# client = Client(processes=False,
#                 threads_per_worker=1,
#                 n_workers=4,
#                 memory_limit='2GB')
# client

# set hyperparams
# How many times to tune in a real application
# we would tune many more times than 2


def seg_char(x):
    """This function segments Chinese characters"""

    x = ' '.join(jieba.cut(x))

    return x




def load_data(filename):
    '''This function loads the data and then it splits the data into labels
    and features.'''
    # dtypes = {'category': 'int', 'text': 'str'}
    df = pd.read_csv(filename)
    # selected = ['category', 'text']
    # selected = ['military_spend', 'content']
    # selected = ['implicit_military_force', 'content']
    # selected = ['territory', 'content']
#     df = df.head(n=2499)
    # df['combined'] = df.apply(combine_columns, axis=1)
    # print(sum(df['combined']))
    # selected = ['trade_barrier', 'content']
    selected = ['national_humiliation', 'content']
    non_selected = list(set(df.columns) - set(selected))

    df = df.drop(non_selected, axis=1)
    df = df.dropna(axis=0, how='any', subset=selected)
    df = df.reindex(np.random.permutation(df.index))

    labels = df[selected[0]].tolist()

    x_raw = df[selected[1]].apply(lambda x: seg_char(x).split(' '))

    return x_raw, labels


# load data
# DATA_PATH = "../../data/more_restrict_humil_5000_jz_till_2500.csv"
DATA_PATH = "more_restrict_humil_5000_jz_till_5000.csv"
X, y = load_data(DATA_PATH)
print(len(X))
X, y = np.array(X), np.array(y)
print("total examples %s" % len(y))

# Create model
# class_weight='balanced' penializes for missed
# classifications in inverse proportion to
# the prevalence of each class


# def count_vect(x):
#     CountVectorizer(analx))
    

svc = Pipeline([
#     ("tfidf_vectorizer", TfidfVectorizer(analyzer=lambda x: x)),
    ("count_vectorizer", CountVectorizer(analyzer=lambda x: x)),
    # ('scale', MaxAbsScaler()),
    (
        "linear svc",
#         SVC(C=0.02, kernel="linear", random_state=7, class_weight='balanced'))
        SVC(C=0.05145454545454546, kernel="linear", random_state=7, class_weight='balanced'))
        # SVC(kernel="linear", random_state=7, class_weight='balanced'))
])

# Make dictionary to store scores
# scoring = {
#     'acc': 'accuracy',
#     'prec_macro': 'precision_macro',
#     'rec_macro': 'recall_macro',
#     'auc': 'roc_auc_score'
# }

# Cross validate
# clf = dcv.RandomizedSearchCV(
#     svc,
#     tuned_parameters,
#     n_iter=n_iter,
#     cv=5,
#     scoring=scoring,
#     refit="rec_macro",
#     # verbose=1,
#     return_train_score=False,
#     scheduler=client)

# clf.fit(X_train, y_train)

X_train, X_test, y_train, y_test = train_test_split(X,
                                                    y,
                                                    stratify=y,
                                                    random_state=7,
                                                    test_size=0.2)


svc.fit(X_train, y_train)

# for param, score in zip(clf.cv_results_['params'], clf.cv_results_['acc'],
#                         clf.cv_results_['prec_macro'],
#                         clf.cv_results_['rec_macro']):
#     print(param, score)

# with open(results_path + '/tune_svm_humil.csv', 'w',
#           newline='') as resultFile:
#     wr = csv.writer(resultFile)
#     wr.writerow(scores.keys())
#     wr.writerows(zip(*scores.values()))

# print("the best model is" + str(clf.best_params_))

# score = clf.best_estimator_.score(X_test, y_test)

# print("On the test data it got a accuracy score of " + str(score))


target_names = ['Not Humil', "Humil"]
svc.fit(X_train, y_train)

score = svc.score(X_test, y_test)

print("On the test data it got a accuracy score of " + str(score))

y_pred = svc.predict(X_test)

recall_score = recall_score(y_test, y_pred)

print("On the test data it got a recall score of " + str(recall_score))

precision_score = precision_score(y_test, y_pred)

print("precision score of" + str(precision_score))

roc_auc_score = roc_auc_score(y_test, y_pred)

print("auc score of" + str(roc_auc_score))



score_results = {"acc": score, "auc": roc_auc_score,
                    "percision": precision_score, "recall": recall_score}

score_results = pd.DataFrame(data = score_results, index = [0])

score_results.to_csv(score_path, index = False)


###
# Cross validate
###



# Make dictionary to store scores
# scoring = {
#     'acc': 'accuracy',
#     'prec_micro': 'precision_micro',
#     'rec_micro': 'recall_micro',
#     'auc': 'roc_auc'
# }

scoring = {
    'acc': 'accuracy',
    'prec': 'precision',
    'rec': 'recall',
    'auc': 'roc_auc'
}

# Run 5-fold cross validation
scores = cross_validate(svc, X, y, cv=5, scoring=scoring)


# Save cross validation scores to csv file
with open(cv_score_path, 'w',
          newline='') as resultFile:
    wr = csv.writer(resultFile)
    wr.writerow(scores.keys())
    wr.writerows(zip(*scores.values()))



