#!/usr/bin/env python
# coding: utf-8

# In[1]:


import pandas as pd
import numpy as np
import jieba
from sklearn.feature_extraction.text import CountVectorizer, TfidfVectorizer
from sklearn.pipeline import Pipeline
from sklearn.svm import SVC
from sklearn.model_selection import cross_validate
import csv
# import dask_ml.model_selection as dcv
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import MaxAbsScaler, OneHotEncoder, MinMaxScaler
from sklearn.pipeline import FeatureUnion
from sklearn.base import BaseEstimator, TransformerMixin
from sklearn.metrics import recall_score, precision_score, roc_auc_score, accuracy_score


# from joblib import dump, load
# import dill as pickle


from datetime import datetime
import time


# set seed
np.random.seed(0)


# set path to save csv results
score_path = '../data/scores/territory_scores.csv'

cv_score_path = '../data/scores/territory_cv_scores.csv'


def seg_char(x):
    """This function segments Chinese characters"""

    x = ' '.join(jieba.cut(x))

    return x



class ItemSelector(BaseEstimator, TransformerMixin):
    """For data grouped by feature, select subset of data at a provided key.

    The data is expected to be stored in a 2D data structure, where the first
    index is over features and the second is over samples.  i.e.

    >> len(data[key]) == n_samples

    Please note that this is the opposite convention to scikit-learn feature
    matrixes (where the first index corresponds to sample).

    ItemSelector only requires that the collection implement getitem
    (data[key]).  Examples include: a dict of lists, 2D numpy array, Pandas
    DataFrame, numpy record array, etc.

    >> data = {'a': [1, 5, 2, 5, 2, 8],
               'b': [9, 4, 1, 4, 1, 3]}
    >> ds = ItemSelector(key='a')
    >> data['a'] == ds.transform(data)

    ItemSelector is not designed to handle data grouped by sample.  (e.g. a
    list of dicts).  If your data is structured this way, consider a
    transformer along the lines of `sklearn.feature_extraction.DictVectorizer`.

    Parameters
    ----------
    key : hashable, required
        The key corresponding to the desired value in a mappable.
    """

    def __init__(self, key, reshape=False):
        self.key = key
        self.reshape = reshape

    def fit(self, x, y=None):
        return self
    
    def date_to_year(self, x):
        try:
            year = datetime.strptime(x, '%Y-%m-%d %H:%M').year
        except Exception as e:
            year = 0
        return year
        

    def transform(self, data_dict):
        if self.reshape == "date_time":
            # some datetimes don't conform to %Y-%m-%d %H:%M so replace with 0 instead of failing
#             x = data_dict[self.key].apply(lambda x: datetime.strptime(x, '%Y-%m-%d %H:%M').year)
              x = data_dict[self.key].apply(self.date_to_year)
              return np.array(x).reshape(-1,1)
#             return convert_to_timestamp(x)
            
        if self.reshape:
            return np.array(data_dict[self.key]).reshape(-1,1)
        return data_dict[self.key]

    
    
def load_data(filename):
    '''This function loads the data and adds the political posts to
    a sample of 45,000 non-political posts. Then it splits the data into labels
    and features.'''
    # dtypes = {'category': 'int', 'text': 'str'}
    df = pd.read_csv(filename)
    # selected = ['category', 'text']
    # selected = ['military_spend', 'content']
    label = 'territory'
    #     selected = ['implicit_military_force', 'content']
    # selected = ['territory', 'content']
#     df = df.head(n=3499)
#     df['combined'] = df.apply(combine_columns, axis=1)
    all_cols = ['uid', 'numlikes', 'numreposts','source','content', 'created_at', label]
    feature_cols = ['uid', 'numlikes', 'numreposts','source','created_at', 'content']
    df = df.loc[:, all_cols]
    df = df.dropna(axis=0, how='any')
    print('after droping shape is ' + str(df.shape))
    y = df[label].tolist()
    df = df.loc[:, feature_cols]

    df['content'] = df['content'].apply(lambda x: seg_char(x).split(' '))

    return df, y


# load data
DATA_PATH = "ir_exclude4_1-5000_jz_coding_till_5000.csv"

df, y = load_data(DATA_PATH)


# print(len(X))

print("df shape is " + str(df.shape))

X_train, X_test, y_train, y_test = train_test_split(df,
                                                    y,
                                                    stratify=y,
                                                    random_state=7,
                                                    test_size=0.2)

# Create model
# class_weight='balanced' penializes for missed
# classifications in inverse proportion to
# the prevalence of each class

transformer = FeatureUnion([
#                 ('source',
#                  Pipeline([
#                      ('selector', ItemSelector(key='source', reshape=True)),
#                      ('one_hot', OneHotEncoder(handle_unknown="ignore", sparse=True)),
#                  ])),
#                 ('numlikes',
#                  Pipeline([
#                      ('selector', ItemSelector(key='numlikes', reshape=True)),
#                      ('min_max', MinMaxScaler(feature_range=(-1, 1))),
#                  ])),
#                 ('num_reposts',
#                  Pipeline([
#                      ('selector', ItemSelector(key='numreposts', reshape=True)),
#                      ('min_max', MinMaxScaler(feature_range=(-1, 1))),
#                  ])),
#                 ('created_at',
#                  Pipeline([('selector', ItemSelector(key='created_at', reshape="date_time")),
#                      ('one_hot', OneHotEncoder(handle_unknown="ignore", sparse=True)),
#                  ])),

                ('content',
                 Pipeline([
                     ('selector', ItemSelector(key='content')),
                     ('tfidf_vectorizer',
                      TfidfVectorizer(analyzer=lambda x: x)),
                 ])),
            ])

pipeline = Pipeline([
    ('transformer', transformer),
#     ('linear svc', SVC(kernel='linear', C = 1.9272727272727272, random_state=7, class_weight='balanced')),
    ('linear svc', SVC(kernel='linear', C = 0.43333333333333335, random_state=7, class_weight='balanced')),
    ])


pipeline.fit(X_train, y_train)

score = pipeline.score(X_test, y_test)

print("On the test data it got a accuracy score of " + str(score))

y_pred = pipeline.predict(X_test)

recall_score = recall_score(y_test, y_pred)

print("On the test data it got a recall score of " + str(recall_score))

precision_score = precision_score(y_test, y_pred)

print("precision score of" + str(precision_score))

roc_auc_score = roc_auc_score(y_test, y_pred)

print("auc score of" + str(roc_auc_score))






score_results = {"acc": score, "auc": roc_auc_score,
                    "percision": precision_score, "recall": recall_score}

score_results = pd.DataFrame(data = score_results, index = [0])

score_results.to_csv(score_path, index = False)


###
# Cross validate
###

# Make dictionary to store scores
scoring = {
    'acc': 'accuracy',
    'prec': 'precision',
    'rec': 'recall',
    'auc': 'roc_auc'
}


# Run 5-fold cross validation
scores = cross_validate(pipeline, df, y, cv=5, scoring=scoring)


# Save cross validation scores to csv file
with open(cv_score_path, 'w',
          newline='') as resultFile:
    wr = csv.writer(resultFile)
    wr.writerow(scores.keys())
    wr.writerows(zip(*scores.values()))



