from google.cloud import bigquery
import math
import sys
import numpy as np
from scipy.stats import t
import pandas as pd


def get_intercept(dataset, model_name, client):
    query = ("SELECT weight FROM "
             "ml.weights(MODEL `" + dataset + "." + model_name +"`) WHERE processed_input = \"__INTERCEPT__\"")
    query_job = client.query(query)
    result = query_job.result()
    for row in result:
        return row.weight


# def rsquared(data, coeffs, iv, controls, dataset, client, model_name, reuse_correlation=None):
#     if reuse_correlation is not None:
#         # This allows reusing regression of controls on IV if it is identical to a previous model which saves money
#         query = ("SELECT r2_score FROM ML.EVALUATE(MODEL `" + dataset + "." + reuse_correlation +"`)")
#         query_job = client.query(query)
#         print("reusing treatment and control correlations from " + reuse_correlation)
#         result = query_job.result()
#         for row in result:
#             coeffs[iv]['r2_score'] = row.r2_score
#         return coeffs
#     else:
#         query = ("CREATE OR REPLACE MODEL `" + dataset + ".treatment_correlations_" + model_name + "` "
#              "TRANSFORM(label," + ",".join(controls) + ") "
#             "OPTIONS (model_type='linear_reg',DATA_SPLIT_METHOD='NO_SPLIT') AS "
#             "SELECT " + iv + " AS label, " + ",".join(controls) + " FROM `" + data + "` ")
#         query_job = client.query(query)
#         result = query_job.result()
#         query = ("SELECT r2_score FROM ML.EVALUATE(MODEL `" + dataset + ".treatment_correlations_" + model_name + "`)")
#         query_job = client.query(query)
#         result = query_job.result()
#         for row in result:
#             coeffs[iv]['r2_score'] = row.r2_score
#         return coeffs


def rsquared_request(data, iv, controls, dataset, client, model_name, reuse_correlation=None):
    query = ("CREATE OR REPLACE MODEL `" + dataset + "." +  model_name + "` "
             "TRANSFORM(label," + ",".join(controls) + ") "
            "OPTIONS (model_type='linear_reg',DATA_SPLIT_METHOD='NO_SPLIT') AS "
            "SELECT " + iv + " AS label, " + ",".join(controls) + " FROM `" + data + "` ")
    return(query)
        # query_job = client.query(query)
        # result = query_job.result()
        # query = ("SELECT r2_score FROM ML.EVALUATE(MODEL `" + dataset + ".treatment_correlations_" + model_name + "`)")
        # query_job = client.query(query)
        # result = query_job.result()
        # for row in result:
        #     coeffs[iv]['r2_score'] = row.r2_score
        # return coeffs

def get_rsquared(data, coeffs, iv, controls, dataset, client, model_name, reuse_correlation):
    query = ("SELECT r2_score FROM ML.EVALUATE(MODEL `" + dataset + "." + reuse_correlation +"`)")
    query_job = client.query(query)
    print("reusing treatment and control correlations from " + reuse_correlation)
    result = query_job.result()
    for row in result:
        coeffs[iv]['r2_score'] = row.r2_score
    return coeffs


def rmse(model, client, dataset):
    query = ("SELECT sqrt(mean_squared_error) as root_mean_squared_error "
        "FROM ML.EVALUATE(MODEL `" + dataset + "." + model + "`)")
    query_job = client.query(query)
    result = query_job.result()
    for row in result:
        return row.root_mean_squared_error

def standef(model, client, dataset):
    query = ("SELECT input, stddev FROM "
        "ml.feature_info(MODEL `" + dataset + "." + model + "`)")
    query_job = client.query(query)
    result = query_job.result()
    coeffs = {}
    for row in result:
            coeffs[row.input] = {'stddev': row.stddev}
    return coeffs

def df_correction(dataset, model_name, client):
    query = "SELECT ANY_VALUE(subquery.num_inputs) as num_inputs, ANY_VALUE(subquery.categories) as num_categories, " \
    + "SUM(subquery.num_inputs + subquery.categories) as total_params " \
    + "FROM (SELECT COUNT(input) as num_inputs, SUM(category_count) as categories FROM ML.FEATURE_INFO(MODEL " \
    + "`" + dataset + "." + model_name + "`)) as subquery"
    query_job = client.query(query)
    result = query_job.result().to_dataframe()
    if pd.isna(result["total_params"].iloc[0]):
        print("There are no categorical variables detected so using number of inputs to calc degrees of freedom")
        return(result["num_inputs"].iloc[0] + 1)
    else:
        return(result["total_params"].iloc[0])


def standarderror(root_mean, sqrtn, coeffs, iv, controls):
    if not controls:
        corr = 1
    else:
        corr = math.sqrt(1-coeffs[iv]['r2_score'])
#     error = root_mean/(coeffs[iv]['stddev']*corr)
#     error = error * (1/ (n - used_degress_of_freedom))
    error = root_mean/(sqrtn*coeffs[iv]['stddev']*corr)
    coeffs[iv]['standard error'] = error


def get_n(data, iv, client):
    query = "SELECT COUNT(" + iv + ") as n FROM `" + data +"`"
#     print(query)
    query_job = client.query(query)
    result = query_job.result().to_dataframe()
    return(result["n"].iloc[0])
    
def coefficients(coeffs, dataset, model_name, client):
    query = ("SELECT input, weight FROM (ML.FEATURE_INFO(MODEL `" + dataset + "." + model_name + "`) LEFT JOIN ML.WEIGHTS(MODEL `" + dataset + "." + model_name + "`) ON  input = processed_input)")
    query_job = client.query(query)
    result = query_job.result()
    for row in result:
            coeffs[row.input]['coefficient'] = row.weight
    

    
# def regression(dataset, model_name, iv, dv, controls, client, data):
#     query = ("CREATE OR REPLACE MODEL `" + dataset + "." + model_name +"` " \
#                  + "TRANSFORM(label,"  + iv +  "," + ",".join(controls) + ") " \
#                  + " OPTIONS(model_type='linear_reg',DATA_SPLIT_METHOD='NO_SPLIT') AS " \
#                  + "SELECT " \
#                  + dv + " AS label," \
#                  + iv + "," \
#                  + ",".join(controls) \
#                  + " FROM `"+ data +"`")
#     query_job = client.query(query)
#     print("The regression query is " + query)
#     # # must wait for query to complete
#     query_job.result()

def regression_request(dataset, model_name, iv, dv, controls, client, data):
    query = ("CREATE OR REPLACE MODEL `" + dataset + "." + model_name +"` " \
                 + "TRANSFORM(label,"  + iv +  "," + ",".join(controls) + ") " \
                 + " OPTIONS(model_type='linear_reg',DATA_SPLIT_METHOD='NO_SPLIT') AS " \
                 + "SELECT " \
                 + dv + " AS label," \
                 + iv + "," \
                 + ",".join(controls) \
                 + " FROM `"+ data +"`")
    return(query)
    # query_job = client.query(query)
    # print("The regression query is " + query)
    # # must wait for query to complete
    # query_job.result()
