from google.cloud import bigquery
import math
import sys
import numpy as np
from scipy.stats import t
import pandas as pd
# import custom functions
from regression_functions import get_intercept, get_rsquared, rsquared_request, rmse, standef, df_correction, standarderror, get_n, coefficients, regression_request

#this is needed for batch requests
from concurrent.futures import ThreadPoolExecutor as PoolExecutor


# job_config = bigquery.QueryJobConfig(use_query_cache=False)

def run_it(query):
  # print(request.execute())
  print("The regression query is " + query)
  query_job = client.query(query)
  # query_job.add_done_callback(dummy_callback)
  query_job.result()
  #wait until query is complete before continuing


# def dummy_callback(future):
#     global jobs_done
#     jobs_done[future.job_id] = True


# n_list =[]

# coef_list = []

# se_list = []


# def get_post_info():
#     n = get_n(data = data, iv = iv, client=client)
#     n_list.append(n)
#     used_degress_of_freedom = df_correction(dataset = dataset, model_name = model_name, client=client)
#     sqrtn = math.sqrt(n - used_degress_of_freedom)
#     root_mean = rmse(model_name, client=client, dataset=dataset)
#     coeffs = standef(model_name, client=client, dataset=dataset)




client = bigquery.Client()

bv_model_list = []        

model_list = []

bv_query_list = []

query_list = []

bv_results_path_list = []

results_path_list = []



    
# Provide data and model info for multivariate combined_force DV

dataset = 'bigquery-226322.bq_catching_fire.'
data = 'bigquery-226322.bq_catching_fire.post_level'
results_dir = "../data/"




####
# Run bivariate models first
####


# bv force model

model_name = "post_level_bv_humil_force"

bv_model_list.append(model_name)

bv_force_results_csv_path = (results_dir + model_name + "_results.csv")

bv_results_path_list.append(bv_force_results_csv_path)

iv = "national_humiliation"
dv = "combined_force"
controls = []



bv_force_model_request = regression_request(dataset=dataset, model_name=model_name, iv=iv, 
    dv=dv, controls=controls, client=client, data=data)
bv_query_list.append(bv_force_model_request)

# bv territory model


model_name = "post_level_bv_humil_territory"

bv_model_list.append(model_name)

bv_territory_results_csv_path = (results_dir + model_name + "_results.csv")

bv_results_path_list.append(bv_territory_results_csv_path)

dv = "territory"
controls = []



bv_territory_model_request = regression_request(dataset=dataset, model_name=model_name, iv=iv, 
    dv=dv, controls=controls, client=client, data=data)
bv_query_list.append(bv_territory_model_request)


# bv trade model


model_name = "post_level_bv_humil_trade_barrier"

bv_model_list.append(model_name)

bv_trade_barrier_results_csv_path = (results_dir + model_name + "_results.csv")

bv_results_path_list.append(bv_trade_barrier_results_csv_path)

dv = "trade_barrier"
controls = []



bv_trade_barrier_model_request = regression_request(dataset=dataset, model_name=model_name, iv=iv, 
    dv=dv, controls=controls, client=client, data=data)
bv_query_list.append(bv_trade_barrier_model_request)


# bv income model

model_name = "post_level_bv_humil_income_ineq"

bv_model_list.append(model_name)

bv_income_ineq_results_csv_path = (results_dir + model_name + "_results.csv")

bv_results_path_list.append(bv_income_ineq_results_csv_path)

dv = "income_ineq"
controls = []



bv_income_ineq_model_request = regression_request(dataset=dataset, model_name=model_name, iv=iv, 
    dv=dv, controls=controls, client=client, data=data)
bv_query_list.append(bv_income_ineq_model_request)


# this will run every model in the bv_querylist

with PoolExecutor(max_workers=8) as executor:
  for query in executor.map(run_it,bv_query_list):
    pass

# print(query)

# calc se for bv models

# this is the same for all models in the script because there is no missing data
n = get_n(data = data, iv = iv, client=client)

for i in range(len(bv_model_list)):
    used_degress_of_freedom = df_correction(dataset = dataset, model_name = bv_model_list[i], client=client)
    print("df of regression is " + str(n - used_degress_of_freedom))
    # Calcu standard error for iv
    sqrtn = math.sqrt(n - used_degress_of_freedom)
    root_mean = rmse(bv_model_list[i], client=client, dataset=dataset)
    coeffs = standef(bv_model_list[i], client=client, dataset=dataset)
    standarderror(root_mean, sqrtn, coeffs, iv, controls=controls)
    intercept = get_intercept(dataset, bv_model_list[i], client=client)
    print("The intercept is " + str(intercept))
    coeff_results = coefficients(coeffs, dataset, bv_model_list[i], client=client)
    # Show results in terminal
    print(iv + " coefficient: " + str(coeffs[iv]['coefficient']))
    print(iv + " standard error: " + str(coeffs[iv]['standard error']))
    # save csv of iv/treatment results
    treatment_results = {"coefficient": coeffs[iv]['coefficient'], "standard_error": coeffs[iv]['standard error'],
                    "observations": n}
    treatment_results = pd.DataFrame(data = treatment_results, index = [0])
    treatment_results.to_csv(bv_results_path_list[i], index = False)



####
# Multivariate models
####


model_name = "post_level_mv_humil_force"
model_list.append(model_name)

force_results_csv_path = (results_dir + model_name + "_results.csv")
results_path_list.append(force_results_csv_path)

dv = "combined_force"
controls = ["mid", "aiguo", "income_ineq", "content_length", "uid", "date"]



force_model_request = regression_request(dataset=dataset, model_name=model_name, iv=iv, dv=dv, controls=controls, client=client, data=data)
query_list.append(force_model_request)




# for territory model request

model_name = "post_level_mv_humil_territory"
model_list.append(model_name)


territory_results_csv_path = (results_dir + model_name + "_results.csv")
results_path_list.append(territory_results_csv_path)


# iv = "national_humiliation"
dv = "territory"




territory_model_request = regression_request(dataset=dataset, model_name=model_name, iv=iv, dv=dv, controls=controls, client=client, data=data)
query_list.append(territory_model_request)


# For trade_barrier request
model_name = "post_level_mv_humil_trade_barrier"
model_list.append(model_name)


trade_results_csv_path = (results_dir + model_name + "_results.csv")
results_path_list.append(trade_results_csv_path)

# iv = "national_humiliation"
dv = "trade_barrier"




trade_model_request = regression_request(dataset=dataset, model_name=model_name, iv=iv, dv=dv, controls=controls, client=client, data=data)

query_list.append(trade_model_request)


# Provide data and model info for multivariate income_ineq dv
model_name = "post_level_mv_humil_income_ineq"
model_list.append(model_name)


placebo_model_name = model_name
income_ineq_results_csv_path = (results_dir + model_name + "_results.csv")

results_path_list.append(income_ineq_results_csv_path)

# iv = "national_humiliation"
dv = "income_ineq"
income_controls = ["mid", "aiguo", "content_length", "uid", "date"]



income_ineq_model_request = regression_request(dataset=dataset, model_name=model_name, iv=iv, dv=dv,
 controls=income_controls, client=client, data=data)

query_list.append(income_ineq_model_request)



######
#This gets treatment and control correlations
######

#do not add to model list
model_name = "post_level_treatment_cor"

treatment_corr_model_name = model_name

treatment_correlation_model_request = rsquared_request(data=data, iv=iv,
 controls=controls, dataset=dataset, client=client, model_name=model_name)

query_list.append(treatment_correlation_model_request)

###
#This gets placebo and control correlations
###

#do not add to model list
model_name = "post_level_income_cor"
placebo_corr_model_name = model_name


income_correlation_model_request = rsquared_request(data=data, iv=iv,
 controls=income_controls, dataset=dataset, client=client, model_name=model_name)

query_list.append(income_correlation_model_request)



# this will run every model in the model list

with PoolExecutor(max_workers=8) as executor:
  for query in executor.map(run_it,query_list):
    pass

# print(query)




# This only runs for 1 model at a time but does not run any new regressions
# so it should not take too long
for i in range(len(model_list)):
    used_degress_of_freedom = df_correction(dataset = dataset, model_name = model_list[i], client=client)
    sqrtn = math.sqrt(n - used_degress_of_freedom)
    root_mean = rmse(model_list[i], client=client, dataset=dataset)
    coeffs = standef(model_list[i], client=client, dataset=dataset)
    if model_list[i] == placebo_model_name:
        coeffs = get_rsquared(data=data, coeffs=coeffs, iv=iv, controls=income_controls, 
        dataset=dataset, client=client, model_name=model_list[i],
        reuse_correlation = placebo_corr_model_name)
    else:
        coeffs = get_rsquared(data=data, coeffs=coeffs, iv=iv, controls=controls, 
        dataset=dataset, client=client, model_name=model_list[i],
        reuse_correlation = treatment_corr_model_name)

    standarderror(root_mean, sqrtn, coeffs, iv, controls=controls)
    intercept = get_intercept(dataset, model_list[i], client=client)
    print("The intercept is " + str(intercept))
    coeff_results = coefficients(coeffs, dataset, model_list[i], client=client)
    # Show results in terminal
    print(iv + " coefficient: " + str(coeffs[iv]['coefficient']))
    print(iv + " standard error: " + str(coeffs[iv]['standard error']))
    for coeff in coeffs.keys():
        if coeff != iv:
            print(coeff + " coefficient: " + str(coeffs[coeff]['coefficient']))
    # save csv of iv/treatment results
    treatment_results = {"coefficient": coeffs[iv]['coefficient'], "standard_error": coeffs[iv]['standard error'],
    "observations": n}
    treatment_results = pd.DataFrame(data = treatment_results, index = [0])
    treatment_results.to_csv(results_path_list[i], index = False)





####################

###everything below here must be done separately for each model
####################


# Get regression meta info
# print("number of obs in dataset is " + str(n))

# used_degress_of_freedom = df_correction(dataset = dataset, model_name = model_name, client=client)
# print("df of regression is " + str(n - used_degress_of_freedom))

# Calcu standard error for iv

# sqrtn = math.sqrt(n - used_degress_of_freedom)
# root_mean = rmse(model_name, client=client, dataset=dataset)
# coeffs = standef(model_name, client=client, dataset=dataset)

# print(coeffs)

# print(type(coeffs))

# #this one runs regressions
# if controls:
#     print("controls included")
#     coeffs = rsquared(data=data, coeffs=coeffs, iv=iv, controls=controls, dataset=dataset, client=client, model_name=model_name)



# standarderror(root_mean, sqrtn, coeffs, iv, controls=controls)

# intercept = get_intercept(dataset, model_name, client=client)

# # standard_error_intercept = intercept_standarderror(dataset, model_name, root_mean, sqrtn)


# print("The intercept is " + str(intercept))

# coeff_results = coefficients(coeffs, dataset, model_name, client=client)



# # Show results in terminal

# print(iv + " coefficient: " + str(coeffs[iv]['coefficient']))
# print(iv + " standard error: " + str(coeffs[iv]['standard error']))


# for coeff in coeffs.keys():
#     if coeff != iv:
#         print(coeff + " coefficient: " + str(coeffs[coeff]['coefficient']))

# # save csv of iv/treatment results
# treatment_results = {"coefficent": coeffs[iv]['coefficient'], "standard_error": coeffs[iv]['standard error'],
#                     "observations": n}

# treatment_results = pd.DataFrame(data = treatment_results, index = [0])

# treatment_results.to_csv(results_csv_path, index = False)